@@ -126,9 +126,9 @@ class Selve:
 
                 await asyncio.sleep(0.1)
 
-            except serial.SerialException:
+            except (serial.SerialException, IOError) as e:
                 # log message
-                self._LOGGER.error("(Selve Worker): " + 'Serial Port RX error')
+                self._LOGGER.error("(Selve Worker): " + 'Serial Port RX error ' + str(e))
                 self._LOGGER.error("(Selve Worker): " + 'trying to reconnect...')
                 await self.recover()
 
@@ -138,7 +138,33 @@ class Selve:
     # serious trouble
     
 
+    def list_ports(self):
+        available_ports = list_ports.comports()
+        return available_ports
 
+    async def check_port(self, port):
+        if port is not None:
+            try:
+                self._serial = serial.Serial(
+                    port=port,
+                    baudrate=115200,
+                    bytesize=serial.EIGHTBITS,
+                    parity=serial.PARITY_NONE,
+                    stopbits=serial.STOPBITS_ONE,
+                    xonxoff=False,
+                    rtscts=False,
+                    dsrdtr=False)
+
+                if await self.pingGateway(fromConfigFlow=True):
+                    self._serial.close()
+                    return True
+            except (serial.SerialException, IOError) as e:
+                self._LOGGER.debug("Configured port not valid! " + str(e))
+                return False
+            except Exception as e:
+                self._LOGGER.error("Unknown exception: " + str(e))
+                return False
+        return False
 
 
     async def setup(self, discover=False, fromConfigFlow=False):
@@ -167,7 +193,7 @@ class Selve:
                             await self.discover()
                         await self.startWorker()
                     return
-            except serial.SerialException as e:
+            except (serial.SerialException, IOError) as e:
                 self._LOGGER.debug("Configured port not valid! " + str(e))
             except Exception as e:
                 self._LOGGER.error("Unknown exception: " + str(e))
@@ -237,7 +263,7 @@ class Selve:
 
                 if await self.pingGatewayFromWorker():
                     return
-            except serial.SerialException as e:
+            except (serial.SerialException, IOError) as e:
                 self._LOGGER.debug("(Selve Worker): " + "Configured port not valid, maybe it has changed, trying other ports...")
             except Exception as e:
                 self._LOGGER.error("(Selve Worker): " + "Unknown exception: " + str(e))
@@ -356,7 +382,7 @@ class Selve:
             #always sleep after writing
             await asyncio.sleep(0.5)
 
-        except SerialException as se:
+        except (serial.SerialException, IOError) as se:
             self._LOGGER.info('Serial error, trying to reconnect once... ' + str(se))
             await self.recover()
 
@@ -681,7 +707,7 @@ class Selve:
     async def _executeCommandSyncWithResponse(self, command: Command):
         async with self._writeLock:
             async with self._readLock:
-                if not self._serial.is_open:
+                if self._serial is not None and not self._serial.is_open:
                     self._serial.open()
                 await self._sendCommandToGateway(command)
                 start_time = time.time()
